/*
 *
 *  Copyright (C) 1996-2000, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Sandra von Gehlen
 *
 *  Purpose: Modality Worklist Database
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 2000/03/08 16:37:47 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/include/wrklstdb.h,v $
 *  CVS/RCS Revision: $Revision: 1.13 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#ifndef WRKLSTDB_H
#define WRKLSTDB_H

#include "osconfig.h"

#include <stdio.h>
#include <string.h>

BEGIN_EXTERN_C
/* This #if code is suggested by the gnu autoconf documentation */
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#define NAMLEN(dirent) strlen((dirent)->d_name)
#else
#define dirent direct
#define NAMELEN(dirent) (dirent)->d_namlen
#ifdef HAVE_SYS_NDIR_H
#include <sys/ndir.h>
#endif
#ifdef HAVE_SYS_DIR_H
#include <sys/dir.h>
#endif
#ifdef HAVE_NDIR_H
#include <ndir.h>
#endif
#endif
#ifdef HAVE_IO_H
#include <io.h>
#endif
END_EXTERN_C

#include "dctk.h"
#include "dcdebug.h"
#include "dcvm.h"
#include "dcstack.h"
#include "dclist.h"
#include "dcvrat.h"


enum WDBStatus {
    WDB_Refused_OutOfResorces = 0xa700,
    WDB_Failed_IdentifierDoesNotMatchSOPClass = 0xa900,
    WDB_Failed_UnableToProcess = 0xc000,
    WDB_Cancel = 0xfe00,
    WDB_Success = 0x0000,
    WDB_Pending = 0xff00,
    WDB_Pending_Warning = 0xff01
};

class WorklistDB
{
private:
    const char *dirpath;
#ifndef HAVE__FINDFIRST
    DIR *dirp;
#endif
    DcmDataset *identifiers;
    DcmList *objlist;
    DcmAttributeTag *errorElements;
    DcmAttributeTag *offendingElements;
    DcmLongString *errorComment;
    int fd;
    OFBool foundUnsupportedOptionalKey;
    OFBool failInvalidQuery;
    OFBool verbose;

    void clearObjectList(void);

    void clearDataset(DcmDataset* identifiers);
    
    void putOffendingElements(DcmTagKey & tag);
    
    void putErrorElements(DcmTagKey & tag);

    OFBool checkMatchingKey(DcmElement *elem);

    OFBool checkWklist(DcmDataset* dsetfile, const char* fname);

    int set_readlock(const char *path);

    int release_readlock();


public:

    WorklistDB(const char* dbDirectoryPath);

    ~WorklistDB();
    
    WDBStatus startFindRequest(DcmDataset& findRequestIdentifiers);

    DcmDataset *nextFindResponse(WDBStatus &rStatus);

    WDBStatus cancelFindRequest();
    
    const DcmAttributeTag* getOffendingElements();
    
    const DcmLongString* getErrorComments();

    OFBool checkIdentifiers(DcmDataset* identifiers);

    void setFailInvalidQuery(OFBool doFailInvalidQuery) { failInvalidQuery = doFailInvalidQuery; }

    OFBool getFailInvalidQuery() { return failInvalidQuery; }

    void setVerbose(OFBool beVerbose) { verbose = beVerbose; }

private:

   /* undefined */ WorklistDB(const WorklistDB&);
   /* undefined */ WorklistDB& operator=(const WorklistDB&);

};


#endif

/*
** CVS Log
** $Log: wrklstdb.h,v $
** Revision 1.13  2000/03/08 16:37:47  meichel
** Updated copyright header.
**
** Revision 1.12  2000/02/02 15:18:37  meichel
** Replaced some #if statements by more robust #ifdef
**
** Revision 1.11  1999/06/10 10:48:20  meichel
** Replaced some #if statements by more robust #ifdef
**
** Revision 1.10  1999/04/26 12:59:16  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.9  1998/07/24 13:29:11  meichel
** changed WorklistDB::nextFindResponse() parameters
**   in order to avoid the use of an implicit assignment
**   operator for class DcmDataset.
**
** Revision 1.8  1997/07/21 09:08:58  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.7  1997/04/18 08:40:39  andreas
** - The put/get-methods for all VRs did not conform to the C++-Standard
**   draft. Some Compilers (e.g. SUN-C++ Compiler, Metroworks
**   CodeWarrier, etc.) create many warnings concerning the hiding of
**   overloaded get methods in all derived classes of DcmElement.
**   So the interface of all value representation classes in the
**   library are changed rapidly, e.g.
**   E_Condition get(Uint16 & value, const unsigned long pos);
**   becomes
**   E_Condition getUint16(Uint16 & value, const unsigned long pos);
**   All (retired) "returntype get(...)" methods are deleted.
**   For more information see dcmdata/include/dcelem.h
**
** Revision 1.6  1997/01/13 15:37:07  hewett
** Added function to perform error checking of maching keys prior
** to a database search.
**
** Revision 1.5  1996/05/02 13:40:56  hewett
** Revised error handling.
**
** Revision 1.4  1996/04/30 13:59:24  savonge
** - more errors corrected
** - added checking of wklist database
**
** Revision 1.3  1996/04/29 19:13:45  hewett
** Added header comment and conditional inclusion of <dirent.h>.
**
*/
